/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.se.AbstractServiceEngine;
import org.ow2.petals.se.eip.patterns.Pattern;

/**
 * Main class of the EIP Component. TODO manage send sync and async in all patterns.
 * @author Adrien Louis - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class EIPComponent extends AbstractServiceEngine {

    /**
     * Map of available patterns
     */
    private Map<String, Pattern> patterns;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.AbstractComponent#doInit()
     */
    @Override
    public void doInit() {
        getLogger().info("Initialize the EIP Service Engine.");
        patterns = new HashMap<String, Pattern>();
        ConfigurationExtensions extensions = this.getComponentExtensions();
        Set<String> patternsNames = extensions.keySet();
        for (Object name : patternsNames) {
            String patternClassName = extensions.get(name);
            try {
                getLogger().finest(
                        "Loading the pattern '" + name + "' implemented by '" + patternClassName
                                + "'");

                Class<?> patternClass = this.getClass().getClassLoader()
                        .loadClass(patternClassName);
                Pattern pattern = (Pattern) patternClass.newInstance();
                pattern.init();
                patterns.put(name.toString(), pattern);
                getLogger().finest("Pattern '" + name + "' has been created.");
            } catch (ClassNotFoundException e) {
                getLogger().warning("Class '" + patternClassName + "' has not been found.");
            } catch (InstantiationException e) {
                getLogger().warning("Pattern '" + name + "' can not be instantiated.");
            } catch (IllegalAccessException e) {
                getLogger().warning("IllegalAccessException " + e.getMessage());
            } catch (ClassCastException e) {
                getLogger().warning(
                        "The class '" + patternClassName
                                + "' does not implement the Pattern interface.");
            }
        }
    }

    /**
     * 
     * @return
     */
    public Map<String, Pattern> getPatterns() {
        return patterns;
    }
}
