/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.async;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.process.async.AsyncContext;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public class CommonAsyncContext extends AsyncContext {

    private int ongoingSubMessages;

    private boolean faultRobust = false;

    private boolean exceptionRobust = false;

    private boolean faultToException = false;

    public static final long DEFAULT_TIME_TO_LIVE = 600000;

    private final String pattern;

    private boolean terminated = false;

    /**
     * The Constructor
     * 
     * @param originalExchange
     * @param timeToLive
     * @param pattern
     * @param faultRobust
     * @param exceptionRobust
     * @param ongoingSubMessages
     * @param faultToException
     */
    public CommonAsyncContext(final Exchange originalExchange, final Long timeToLive,
            final String pattern, final int ongoingSubMessages, final boolean faultRobust,
            final boolean exceptionRobust, final boolean faultToException) {
        super(originalExchange, timeToLive);
        this.pattern = pattern;
        this.ongoingSubMessages = ongoingSubMessages;
        this.faultRobust = faultRobust;
        this.exceptionRobust = exceptionRobust;
        this.faultToException = faultToException;
    }

    /**
     * Decrement and get the number of messages to wait
     * 
     * @return The number of messages to wait
     */
    public synchronized int decrementAndGetOngoingSubMessages() {
        this.ongoingSubMessages--;
        return this.ongoingSubMessages;
    }

    public String getPattern() {
        return this.pattern;
    }

    /**
     * Get the exceptionRobust
     * 
     * @return the exceptionRobust
     */
    public boolean isExceptionRobust() {
        return this.exceptionRobust;
    }

    /**
     * Get the faultRobust
     * 
     * @return the faultRobust
     */
    public boolean isFaultRobust() {
        return this.faultRobust;
    }

    /**
     * Get the faultToException
     * 
     * @return the faultToException
     */
    public boolean isFaultToException() {
        return this.faultToException;
    }

    public boolean isTerminated() {
        return this.terminated;
    }

    public void setTerminated() {
        this.terminated = true;
    }
}
