/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.se.eip.ExchangeContext;
import org.ow2.petals.se.eip.async.CommonAsyncContext;

/**
 * The pattern interface.
 * @author Adrien Louis - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public interface Pattern {
    /**
     * Initialize the pattern
     * 
     */
    void init();

    /**
     * Process the Pattern.
     * 
     * @param exchange
     *            The exchange to process
     * @param context
     *            The context of the exchange
     */
    boolean processPattern(Exchange exchange, ExchangeContext context);

    /**
     * Process an exchange for an asynchronous pattern.
     * 
     * @param exchange
     *            The exchange to process
     * @param context
     *            The context of the exchange
     * @param asyncContext
     *            The EIP asynchronous context
     * @param expired
     *            true if the asynchronous exchange is expired, false else
     */
    boolean processAsyncPattern(Exchange exchange, ExchangeContext context,
            CommonAsyncContext asyncContext, boolean expired);

    /**
     * Get the pattern name.
     * 
     * @return The name
     */
    String getName();
}
