/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns;

import java.util.List;
import java.util.Vector;

import javax.jbi.messaging.MessagingException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.xpath.XPathExpressionException;

import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.se.eip.EIPConstants;
import org.w3c.dom.Document;

/**
 * Helper to route.
 * @author Frederic Gardes - EBM WebSourcing
 */
public class XPathRouterEngine implements RouterEngine {

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.se.eip.patterns.RouterEngine#retrieveConditions(org.ow2
     * .petals.component.framework.api.configuration.ConfigurationExtensions)
     */
    public final List<String> getXpathParameters(final ConfigurationExtensions extensions) {
        final Vector<String> xpathParameters = new Vector<String>();
        for (final String param : extensions.keySet()) {
            if (EIPConstants.TEST.equals(param)) {
                xpathParameters.add(extensions.get(param));
                break;
            } else if (param.startsWith(EIPConstants.TEST)) {
                final String indexString = param.substring(EIPConstants.TEST.length());
                int index = Integer.valueOf(indexString);
                if (index > xpathParameters.size()) {
                    xpathParameters.setSize(index);
                }
                xpathParameters.set(index - 1, extensions.get(param));
            }
        }

        return xpathParameters;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.RouterEngine#route(java.util.List,
     * org.ow2.petals.component.framework.api.message.Exchange)
     */
    public final int route(final List<String> conditions, final Exchange exchange)
            throws XPathExpressionException, PEtALSCDKException, MessagingException {

        final Document document = SourceUtil.createDocument(
                exchange.getInMessage().getContent());
        exchange.getInMessage().setContent(new DOMSource(document));

        int targetedConsumesListIndentifier = conditions.size();
        for (int i = 0; i < conditions.size(); i++) {
            if (PatternHelper.evaluateXPathCondition(conditions.get(i),
                    document.getDocumentElement())) {
                targetedConsumesListIndentifier = i;
                break;
            }
        }
        return targetedConsumesListIndentifier;
    }

    /**
     * @param conditions
     * @param inMessage
     * @param context
     * @return
     * @throws XPathExpressionException
     * @throws PEtALSCDKException
     * @throws MessagingException
     */
    public final int route(final List<String> conditions, final Document document)
            throws XPathExpressionException, PEtALSCDKException, MessagingException {

        int targetedConsumesListIndentifier = conditions.size();
        for (int i = 0; i < conditions.size(); i++) {
            if (PatternHelper.evaluateXPathCondition(conditions.get(i),
                    document.getDocumentElement())) {
                targetedConsumesListIndentifier = i;
                break;
            }
        }
        return targetedConsumesListIndentifier;
    }
}
