/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.InputStream;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.jbidescriptor.generated.Param;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.se.eip.EIPConstants;
import org.ow2.petals.se.eip.async.CommonAsyncContext;
import org.ow2.petals.se.eip.patterns.mock.MessageExchangeMock;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.XMLComparator;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public abstract class AbstractAggregatorPatternTest extends AbstractTest {

    private final static String VIRTUAL_NAMESPACE = EIPConstants.NAMESPACE + "/virtualForker";

    protected AbstractAggregatorPattern abstractForkerPattern;

    private Document in1;

    private Document out1;

    private Document out2;

    private Document out3;

    private Document out4;

    private Document out5;

    private Document out6;

    private Document out7;

    private Document fault1;

    /**
     * @param name
     */
    public AbstractAggregatorPatternTest() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    @Before
    public void setUp() throws Exception {
        super.setUp();

        DocumentBuilder documentBuilder = null;
        InputStream inputStream = null;
        
        try {
            documentBuilder = DocumentBuilders.takeDocumentBuilder();
            
            // in 1 
            inputStream = AbstractAggregatorPattern.class.getResourceAsStream("/abstractForkerIn1.xml");  
            this.in1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 1
            inputStream = AbstractAggregatorPattern.class
                    .getResourceAsStream("/abstractForkerOut1.xml");
            this.out1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 2
            inputStream = AbstractAggregatorPatternTest.class.getResourceAsStream("/abstractForkerOut2.xml");
            this.out2 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 3
            inputStream = AbstractAggregatorPatternTest.class.getResourceAsStream("/abstractForkerOut3.xml");
            this.out3 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 4
            inputStream = AbstractAggregatorPatternTest.class.getResourceAsStream("/abstractForkerOut4.xml");
            this.out4 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 5
            inputStream = AbstractAggregatorPatternTest.class.getResourceAsStream("/abstractForkerOut5.xml");
            this.out5 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 6
            inputStream = AbstractAggregatorPatternTest.class.getResourceAsStream("/abstractForkerOut6.xml");
            this.out6 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 7
            inputStream = AbstractAggregatorPatternTest.class.getResourceAsStream("/abstractForkerOut7.xml");
            this.out7 = documentBuilder.parse(inputStream);
            inputStream.close();
            // fault 1
            inputStream = AbstractAggregatorPatternTest.class.getResourceAsStream("/abstractForkerFault1.xml");
            this.fault1 = documentBuilder.parse(inputStream);
        } finally {
            if(documentBuilder != null) {
                DocumentBuilders.releaseDocumentBuilder(documentBuilder);
            }
            if(inputStream != null) {
                inputStream.close();
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#tearDown()
     */
    @After
    public void tearDown() throws Exception {
        super.tearDown();
        this.in1 = null;
        this.out1 = null;
        this.out2 = null;
        this.out3 = null;
        this.out4 = null;
        this.out5 = null;
        this.out6 = null;
        this.out7 = null;
        this.fault1 = null;
        this.abstractForkerPattern = null;
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncDOM() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT, 2);
            this.context.setConsumeList(consumesList);
            // source exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setDOMKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), false, false, false);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.abstractForkerPattern,
                    eipAsyncContext);
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.abstractForkerPattern,
                    eipAsyncContext);
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.out3).replace(VIRTUAL_NAMESPACE,
                            this.abstractForkerPattern.getNameSpace()), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncStream() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT, 2);
            this.context.setConsumeList(consumesList);
            // source exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.abstractForkerPattern,
                    eipAsyncContext);
            // exchange 2
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.abstractForkerPattern,
                    eipAsyncContext);
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.out3).replace(VIRTUAL_NAMESPACE,
                            this.abstractForkerPattern.getNameSpace()), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessWithInOnlyConsumesWithDone() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY, 1);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.abstractForkerPattern,
                    eipAsyncContext);
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertTrue(eipAsyncContext.getOriginalExchange().getFault() == null);
            assertTrue(eipAsyncContext.getOriginalExchange().getError() == null);
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.out4).replace(VIRTUAL_NAMESPACE,
                            this.abstractForkerPattern.getNameSpace()), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessWithInOnlyConsumesWithError() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY, 1);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.abstractForkerPattern, eipAsyncContext);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertTrue(eipAsyncContext.getOriginalExchange().getFault() == null);
            assertTrue(eipAsyncContext.getOriginalExchange().getError() != null);
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessWithInOnlyConsumesWithErrorAndExceptionRobustness() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY, 1);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), false, true, false);
            // exchange 1
            final Exception exception = new Exception("test exception");
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, exception, this.abstractForkerPattern,
                    eipAsyncContext);
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertTrue(eipAsyncContext.getOriginalExchange().getFault() == null);
            assertTrue(eipAsyncContext.getOriginalExchange().getError() == null);
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.out5).replace(VIRTUAL_NAMESPACE,
                            this.abstractForkerPattern.getNameSpace()), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessWithRobustInOnlyConsumesWithDone() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.ROBUST_IN_ONLY, 1);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.abstractForkerPattern,
                    eipAsyncContext);
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertTrue(eipAsyncContext.getOriginalExchange().getFault() == null);
            assertTrue(eipAsyncContext.getOriginalExchange().getError() == null);
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.out4).replace(VIRTUAL_NAMESPACE,
                            this.abstractForkerPattern.getNameSpace()), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessWithRobustInOnlyConsumesWithError() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.ROBUST_IN_ONLY, 1);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), false, false, false);
            // exchange 1
            final Exception exception = new Exception("test exception");
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, exception, this.abstractForkerPattern,
                    eipAsyncContext);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertTrue(eipAsyncContext.getOriginalExchange().getFault() == null);
            assertTrue(eipAsyncContext.getOriginalExchange().getError() != null);
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessWithRobustInOnlyConsumesWithErrorAndExceptionRobustness() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.ROBUST_IN_ONLY, 1);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), false, true, false);
            // exchange 1
            final Exception exception = new Exception("test exception");
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, exception, this.abstractForkerPattern,
                    eipAsyncContext);
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertTrue(eipAsyncContext.getOriginalExchange().getFault() == null);
            assertTrue(eipAsyncContext.getOriginalExchange().getError() == null);
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.out5).replace(VIRTUAL_NAMESPACE,
                            this.abstractForkerPattern.getNameSpace()), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessWithRobustInOnlyConsumesWithFault() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.ROBUST_IN_ONLY, 1);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractForkerPattern, eipAsyncContext);
            // the exchange must on active status with a fault
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertTrue(eipAsyncContext.getOriginalExchange().getFault() != null);
            assertTrue(eipAsyncContext.getOriginalExchange().getError() == null);
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(eipAsyncContext.getOriginalExchange().getFault().getContent()));
            assertEquals(XMLPrettyPrinter.prettyPrint(this.fault1), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessWithRobustInOnlyConsumesWithFaultAndFaultRobustness() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.ROBUST_IN_ONLY, 1);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), true, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractForkerPattern, eipAsyncContext);
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertTrue(eipAsyncContext.getOriginalExchange().getError() == null);
            assertTrue(eipAsyncContext.getOriginalExchange().getFault() == null);
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.out6).replace(VIRTUAL_NAMESPACE,
                            this.abstractForkerPattern.getNameSpace()), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessWithInOutConsumesWithFault() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT, 1);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractForkerPattern, eipAsyncContext);
            // the exchange must on active status with a fault
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertTrue(eipAsyncContext.getOriginalExchange().getFault() != null);
            assertTrue(eipAsyncContext.getOriginalExchange().getError() == null);
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(eipAsyncContext.getOriginalExchange().getFault().getContent()));
            assertEquals(XMLPrettyPrinter.prettyPrint(this.fault1), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessWithInOutConsumesWithOut() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT, 1);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.abstractForkerPattern,
                    eipAsyncContext);
            // the exchange must be on active status with an out
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.out7).replace(VIRTUAL_NAMESPACE,
                            this.abstractForkerPattern.getNameSpace()), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.AbstractAggregatorPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessWithInOutConsumesWithFaultAndFaultRobustness() {
        try {
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT, 1);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.abstractForkerPattern.getName(),
                    consumesList.size(), true, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractForkerPattern, eipAsyncContext);
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertTrue(eipAsyncContext.getOriginalExchange().getError() == null);
            assertTrue(eipAsyncContext.getOriginalExchange().getFault() == null);
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.out6).replace(VIRTUAL_NAMESPACE,
                            this.abstractForkerPattern.getNameSpace()), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    private List<Consumes> getConsumesList(final MEPType mepType, final int size) {
        final List<Consumes> consumesList = new CopyOnWriteArrayList<Consumes>();
        QName interfaceName = null;
        QName serviceName = null;
        String endpoint = null;

        interfaceName = new QName("http://petals.ow2.org", "interface");
        serviceName = new QName("http://petals.ow2.org", "serviceConsumer");
        endpoint = "8b32ee51-9b7b-4a09-8805-99733c1efc9a";
        final Consumes consumes = new Consumes();
        consumes.setMep(mepType);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        for (int i = 0; i < size; i++) {
            consumes.setOperation(new QName("http://petals.ow2.org", "test"
                    + Integer.toString(i + 1)));
            consumesList.add(consumes);
        }
        return consumesList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#getParamList()
     */
    @Override
    public List<Param> getParamList() {
        return new CopyOnWriteArrayList<Param>();
    }

}
