/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.InputStream;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.jbidescriptor.generated.Param;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.se.eip.EIPConstants;
import org.ow2.petals.se.eip.async.CommonAsyncContext;
import org.ow2.petals.se.eip.patterns.mock.MessageExchangeMock;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.XMLComparator;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public abstract class AbstractMEPBridgeWithUnicConsumesPatternTest extends AbstractTest {

    private final static String VIRTUAL_NAMESPACE = EIPConstants.NAMESPACE + "/virtualMEPBridge";

    private final static String VIRTUAL_NAME = "virtualMEPBridge";

    protected AbstractMEPBridgeWithUnicConsumesPattern abstractMEPBridgeWithUnicConsumesPattern;

    private Document in1;

    private Document out1;

    private Document out2;

    private Document fault1;

    /**
     * @param name
     */
    public AbstractMEPBridgeWithUnicConsumesPatternTest() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#setUp()
     */
    @Before
    public void setUp() throws Exception {
        super.setUp();
        
        DocumentBuilder documentBuilder = null;
        InputStream inputStream = null;
        
        try {
            documentBuilder = DocumentBuilders.takeDocumentBuilder();
        
            // in 1
            inputStream = AbstractMEPBridgeWithUnicConsumesPattern.class
                    .getResourceAsStream("/abstractMEPBridgeIn1.xml");
            this.in1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 1
            inputStream = AbstractMEPBridgeWithUnicConsumesPattern.class
                    .getResourceAsStream("/abstractMEPBridgeOut1.xml");
            this.out1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 2
            inputStream = AbstractMEPBridgeWithUnicConsumesPattern.class
                    .getResourceAsStream("/abstractMEPBridgeOut2.xml");
            this.out2 = documentBuilder.parse(inputStream);
            inputStream.close();
            // fault 1
            inputStream = AbstractMEPBridgeWithUnicConsumesPattern.class.getResourceAsStream("/abstractMEPBridgeFault1.xml");
            this.fault1 = documentBuilder.parse(inputStream);
        } finally {
            if(documentBuilder != null) {
                DocumentBuilders.releaseDocumentBuilder(documentBuilder);
            }
            if(inputStream != null) {
                inputStream.close();
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#tearDown()
     */
    @After
    public void tearDown() throws Exception {
        super.tearDown();
        this.abstractMEPBridgeWithUnicConsumesPattern = null;
        this.in1 = null;
        this.out1 = null;
        this.out2 = null;
        this.fault1 = null;
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithInOnlyConsumesWithDone() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithInOnlyConsumesWithError() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null,
                    new MessagingException("test exception"),
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("javax.jbi.messaging.MessagingException: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithRobustInOnlyConsumesWithDone() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithRobustInOnlyConsumesWithError() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null,
                    new MessagingException("test exception"),
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("javax.jbi.messaging.MessagingException: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithRobustInOnlyConsumesWithFault() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // InOnly, so no fault thrown, just exceptions
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithRobustInOnlyConsumesWithFaultAndFaultToException() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, true);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // InOnly with fault to exception, so exception
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1), eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithInOutConsumesWithOut() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // InOnly, so no out, just exceptions
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithInOutConsumesWithFault() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // InOnly, so no out, just exceptions
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOnlyWithInOutConsumesWithFaultAndFaultToException() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, true);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // InOnly with fault to exception, so exception
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1), eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithInOnlyConsumesWithDone() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithInOnlyConsumesWithError() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithRobustInOnlyConsumesWithDone() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithRobustInOnlyConsumesWithError() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithRobustInOnlyConsumesWithFault() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNotNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(eipAsyncContext.getOriginalExchange().getFault().getContent()));
            XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithRobustInOnlyConsumesWithFaultAndFaultToException() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, true);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1), eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithInOutConsumesWithOut() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // RobustInOnly, so no out, just exceptions
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithInOutConsumesWithFault() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNotNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(eipAsyncContext.getOriginalExchange().getFault().getContent()));
            XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessRobustInOnlyWithInOutConsumesWithFaultAndFaultToException() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, true);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1), eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOutWithInOnlyConsumesWithDone() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // a default response is set
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNotNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            XMLComparator.isEquivalent(
                    XMLPrettyPrinter
                            .prettyPrint(this.out2)
                            .replace(VIRTUAL_NAMESPACE,
                                    this.abstractMEPBridgeWithUnicConsumesPattern.getNameSpace())
                            .replace(VIRTUAL_NAME,
                                    this.abstractMEPBridgeWithUnicConsumesPattern.getName()),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOutWithInOnlyConsumesWithError() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null,
                    new MessagingException("test exception"),
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            // we can't test if the out message is null. if we invoke the
            // getOutMessage() method...one is created
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("javax.jbi.messaging.MessagingException: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOutWithRobustInOnlyConsumesWithDone() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // a default response is set
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNotNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            XMLComparator.isEquivalent(
                    XMLPrettyPrinter
                            .prettyPrint(this.out2)
                            .replace(VIRTUAL_NAMESPACE,
                                    this.abstractMEPBridgeWithUnicConsumesPattern.getNameSpace())
                            .replace(VIRTUAL_NAME,
                                    this.abstractMEPBridgeWithUnicConsumesPattern.getName()),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOutWithRobustInOnlyConsumesWithError() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null,
                    new MessagingException("test exception"),
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            // we can't test if the out message is null. if we invoke the
            // getOutMessage() method...one is created
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("javax.jbi.messaging.MessagingException: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOutWithRobustInOnlyConsumesWithFault() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            // we can't test if the out message is null. if we invoke the
            // getOutMessage() method...one is created
            assertNotNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(eipAsyncContext.getOriginalExchange().getFault().getContent()));
            XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOutWithRobustInOnlyConsumesWithFaultAndFaultToException() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, true);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            // we can't test if the out message is null. if we invoke the
            // getOutMessage() method...one is created
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1), eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOutWithInOutConsumesWithOut() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNotNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.out1), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOutWithInOutConsumesWithFaultAndFaultToException() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, true);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            // we can't test if the out message is null. if we invoke the
            // getOutMessage() method...one is created
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1), eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessInOutWithInOutConsumesWithFault() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE,
                    this.abstractMEPBridgeWithUnicConsumesPattern.getName(), consumesList.size(),
                    false, false, false);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null,
                    this.abstractMEPBridgeWithUnicConsumesPattern, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            // we can't test if the out message is null. if we invoke the
            // getOutMessage() method...one is created
            assertNotNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(eipAsyncContext.getOriginalExchange().getFault().getContent()));
            XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.VirtualMEPBridgePattern#init()} .
     */
    @Test
    public void testInit() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
        } catch (final Throwable t) {
            fail("An error occurs: " + t.getMessage());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#getParamList()
     */
    @Override
    public List<Param> getParamList() {
        return new CopyOnWriteArrayList<Param>();
    }

    private List<Consumes> getConsumesList(final MEPType mepType) {
        final List<Consumes> consumesList = new CopyOnWriteArrayList<Consumes>();
        QName interfaceName = null;
        QName serviceName = null;
        String endpoint = null;

        interfaceName = new QName("http://petals.ow2.org", "interface");
        serviceName = new QName("http://petals.ow2.org", "serviceConsumer");
        endpoint = "8b32ee51-9b7b-4a09-8805-99733c1efc9a";
        final Consumes consumes = new Consumes();
        consumes.setMep(mepType);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "test1"));
        consumesList.add(consumes);
        return consumesList;
    }
}
