/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.InputStream;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.jbidescriptor.generated.Param;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.se.eip.async.CommonAsyncContext;
import org.ow2.petals.se.eip.patterns.mock.MessageExchangeMock;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.XMLComparator;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public class DispatcherTest extends AbstractTest {

    private Dispatcher dispatcher;

    private Document in1;

    private Document out1;

    private Document out2;

    private Document fault1;

    /**
     * @param name
     */
    public DispatcherTest() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#setUp()
     */
    @Before
    public void setUp() throws Exception {
        super.setUp();
        this.dispatcher = new Dispatcher();
        
        DocumentBuilder documentBuilder = null;
        InputStream inputStream = null;
        
        try {
            documentBuilder = DocumentBuilders.takeDocumentBuilder();
            
            // in 1
            inputStream = RoutingSlip.class.getResourceAsStream("/dispatcherIn1.xml");
            this.in1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 1
            inputStream = RoutingSlip.class.getResourceAsStream("/dispatcherOut1.xml");
            this.out1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 2
            inputStream = RoutingSlip.class.getResourceAsStream("/dispatcherOut2.xml");
            this.out2 = documentBuilder.parse(inputStream);
            inputStream.close();
            // fault 1
            inputStream = RoutingSlip.class.getResourceAsStream("/dispatcherFault1.xml");
            this.fault1 = documentBuilder.parse(inputStream);
        } finally {
            if (documentBuilder != null) {
                DocumentBuilders.releaseDocumentBuilder(documentBuilder);
            }
            if (inputStream != null) {
                inputStream.close();
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#tearDown()
     */
    @After
    public void tearDown() throws Exception {
        super.tearDown();
        this.dispatcher = null;
        this.in1 = null;
        this.out1 = null;
        this.out2 = null;
        this.fault1 = null;
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessDOM() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            // exchange
            final Exchange exchange = this.processDOMTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.dispatcher);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertEquals(2, exchangeList.size());

            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(0)
                            .getInMessage().getContent()))));
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(1)
                            .getInMessage().getContent()))));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessStream() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.dispatcher);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
exchangeList.get(1)
                            .getInMessage().getContent()))));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncDOM() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncStream() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2InOnlyWithDone() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.dispatcher, eipAsyncContext);
            assertEquals(0, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.dispatcher, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2InOnlyWithErrorOnFirst() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.dispatcher, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.dispatcher, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2InOnlyWithErrorOnSecond() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.dispatcher, eipAsyncContext);
            assertEquals(0, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.dispatcher, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2InOutWithOut() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2InOutWithFaultOnFirst() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2InOutWithFaultOnSecond() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2InOutWithFaultOnFirstAndFaultToException() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = true;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertTrue(XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1),
                    eipAsyncContext.getOriginalExchange().getError().getMessage()));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2InOutWithFaultOnSecondAndFaultToException() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = true;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertTrue(XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1),
                    eipAsyncContext.getOriginalExchange().getError().getMessage()));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2RobustInOnlyWithDone() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.ROBUST_IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.dispatcher, eipAsyncContext);
            assertEquals(0, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.dispatcher, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2RobustInOnlyWithErrorOnFirst() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.ROBUST_IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.dispatcher, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.dispatcher, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2RobustInOnlyWithErrorOnSecond() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.ROBUST_IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.dispatcher, eipAsyncContext);
            assertEquals(0, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.dispatcher, eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2RobustInOnlyWithFaultOnFirst() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.ROBUST_IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.dispatcher, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2RobustInOnlyWithFaultOnSecond() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.ROBUST_IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = false;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.dispatcher, eipAsyncContext);
            assertEquals(0, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2RobustInOnlyWithFaultOnFirstAndFaultToException() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.ROBUST_IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = true;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.dispatcher, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertTrue(XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1),
                    eipAsyncContext.getOriginalExchange().getError().getMessage()));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Dispatcher#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
 	public void testProcessAsyncWith2RobustInOnlyWithFaultOnSecondAndFaultToException() {
        try {
            this.dispatcher.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.ROBUST_IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null);
            final boolean faultToException = true;
            final CommonAsyncContext eipAsyncContext = new CommonAsyncContext(originalExchange,
                    CommonAsyncContext.DEFAULT_TIME_TO_LIVE, this.dispatcher.getName(),
                    consumesList.size(), false, false, faultToException);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.dispatcher, eipAsyncContext);
            assertEquals(0, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.dispatcher,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertTrue(XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.fault1),
                    eipAsyncContext.getOriginalExchange().getError().getMessage()));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for {@link org.ow2.petals.se.eip.patterns.Dispatcher#init()}.
     */
    @Test
 	public void testInit() {
        try {
            this.dispatcher.init();
        } catch (final Throwable t) {
            fail("An error occurs: " + t.getMessage());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#getParamList()
     */
    @Override
    public List<Param> getParamList() {
        return new CopyOnWriteArrayList<Param>();
    }

    private List<Consumes> getConsumesList(final MEPType mepType) {
        final List<Consumes> consumesList = new CopyOnWriteArrayList<Consumes>();
        QName interfaceName = null;
        QName serviceName = null;
        String endpoint = null;

        interfaceName = new QName("http://petals.ow2.org", "interface");
        serviceName = new QName("http://petals.ow2.org", "serviceConsumer");
        endpoint = "8b32ee51-9b7b-4a09-8805-99733c1efc9a";
        final Consumes consumes = new Consumes();
        consumes.setMep(mepType);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "test1"));
        consumesList.add(consumes);
        consumes.setOperation(new QName("http://petals.ow2.org", "test2"));
        consumesList.add(consumes);
        return consumesList;
    }
}
