/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.InputStream;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.jbidescriptor.generated.Param;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.se.eip.EIPConstants;
import org.ow2.petals.se.eip.async.CommonAsyncContext;
import org.ow2.petals.se.eip.async.DynamicRouterAsyncContext;
import org.ow2.petals.se.eip.patterns.mock.MessageExchangeMock;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.XMLComparator;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public class DynamicRouterTest extends AbstractTest {

    private DynamicRouter dynamicRouter;

    private Document in1;

    private Document out1;

    private Document out2;

    private Document out3;

    /**
     * @param name
     */
    public DynamicRouterTest() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    @Before
    public void setUp() throws Exception {
        super.setUp();
        this.dynamicRouter = new DynamicRouter();
        
        DocumentBuilder documentBuilder = null;
        InputStream inputStream = null;
        
        try {
            documentBuilder = DocumentBuilders.takeDocumentBuilder();

            // in 1
            inputStream = Router.class.getResourceAsStream("/dynamicRouterIn1.xml");
            this.in1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 1
            inputStream = Router.class.getResourceAsStream("/dynamicRouterOut1.xml");
            this.out1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 2
            inputStream = Router.class.getResourceAsStream("/dynamicRouterOut2.xml");
            this.out2 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 3
            inputStream = Router.class.getResourceAsStream("/dynamicRouterOut3.xml");
            this.out3 = documentBuilder.parse(inputStream);
        } finally {
            if (documentBuilder != null) {
                DocumentBuilders.releaseDocumentBuilder(documentBuilder);
            }
            if (inputStream != null) {
                inputStream.close();
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#tearDown()
     */
    @After
    public void tearDown() throws Exception {
        super.tearDown();
        this.dynamicRouter = null;
        this.in1 = null;
        this.out1 = null;
        this.out2 = null;
        this.out3 = null;
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.DynamicRouter#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessDOM() {
        try {
            this.dynamicRouter.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            // exchange
            final Exchange exchange = this.processDOMTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.dynamicRouter);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(0)
                            .getInMessage().getContent()))));
            assertEquals("router", exchangeList.get(0).getOperation().getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.DynamicRouter#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessStream() {
        try {
            this.dynamicRouter.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.dynamicRouter);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(0)
                            .getInMessage().getContent()))));
            assertEquals("router", exchangeList.get(0).getOperation().getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.DynamicRouter#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.DynamicRouterAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncDOM() {
        try {
            this.dynamicRouter.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setDOMKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null);
            final DynamicRouterAsyncContext eipAsyncContext = new DynamicRouterAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList,
                    this.getConditionList());
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.dynamicRouter,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            assertTrue(this.context.getExchangeList().get(0).isDoneStatus());
            assertTrue(this.context.getExchangeList().get(1).isActiveStatus());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(this.context
                            .getExchangeList().get(1).getInMessage().getContent()))));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.DynamicRouter#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.DynamicRouterAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncStream() {
        try {
            this.dynamicRouter.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null);
            final DynamicRouterAsyncContext eipAsyncContext = new DynamicRouterAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList,
                    this.getConditionList());
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.dynamicRouter,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            assertTrue(this.context.getExchangeList().get(0).isDoneStatus());
            assertTrue(this.context.getExchangeList().get(1).isActiveStatus());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(this.context
                            .getExchangeList().get(1).getInMessage().getContent()))));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.DynamicRouter#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.DynamicRouterAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncTest1DOM() {
        try {
            this.dynamicRouter.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setDOMKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null);
            final DynamicRouterAsyncContext eipAsyncContext = new DynamicRouterAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList,
                    this.getConditionList());
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.dynamicRouter,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            assertTrue(this.context.getExchangeList().get(0).isDoneStatus());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(this.context
                            .getExchangeList().get(1).getInMessage().getContent()))));
            assertEquals("test1", this.context.getExchangeList().get(1).getOperation()
                    .getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.DynamicRouter#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.DynamicRouterAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncTest1Stream() {
        try {
            this.dynamicRouter.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null);
            final DynamicRouterAsyncContext eipAsyncContext = new DynamicRouterAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList,
                    this.getConditionList());
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.dynamicRouter,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            assertTrue(this.context.getExchangeList().get(0).isDoneStatus());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(this.context
                            .getExchangeList().get(1).getInMessage().getContent()))));
            assertEquals("test1", this.context.getExchangeList().get(1).getOperation()
                    .getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.DynamicRouter#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.DynamicRouterAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncTest2DOM() {
        try {
            this.dynamicRouter.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setDOMKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null);
            final DynamicRouterAsyncContext eipAsyncContext = new DynamicRouterAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList,
                    this.getConditionList());
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.dynamicRouter,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            assertTrue(this.context.getExchangeList().get(0).isDoneStatus());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(this.context
                            .getExchangeList().get(1).getInMessage().getContent()))));
            assertEquals("test2", this.context.getExchangeList().get(1).getOperation()
                    .getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.DynamicRouter#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.DynamicRouterAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncTest2Stream() {
        try {
            this.dynamicRouter.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null);
            final DynamicRouterAsyncContext eipAsyncContext = new DynamicRouterAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList,
                    this.getConditionList());
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.dynamicRouter,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            assertTrue(this.context.getExchangeList().get(0).isDoneStatus());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(this.context
                            .getExchangeList().get(1).getInMessage().getContent()))));
            assertEquals("test2", this.context.getExchangeList().get(1).getOperation()
                    .getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.DynamicRouter#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.DynamicRouterAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncDefaultDOM() {
        try {
            this.dynamicRouter.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setDOMKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null);
            final DynamicRouterAsyncContext eipAsyncContext = new DynamicRouterAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList,
                    this.getConditionList());
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out3, null, null, this.dynamicRouter,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            assertTrue(this.context.getExchangeList().get(0).isDoneStatus());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(this.context
                            .getExchangeList().get(1).getInMessage().getContent()))));
            assertEquals("default", this.context.getExchangeList().get(1).getOperation()
                    .getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.DynamicRouter#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.DynamicRouterAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncDefaultStream() {
        try {
            this.dynamicRouter.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null);
            final DynamicRouterAsyncContext eipAsyncContext = new DynamicRouterAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList,
                    this.getConditionList());
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out3, null, null, this.dynamicRouter,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            assertTrue(this.context.getExchangeList().get(0).isDoneStatus());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(this.context
                            .getExchangeList().get(1).getInMessage().getContent()))));
            assertEquals("default", this.context.getExchangeList().get(1).getOperation()
                    .getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.DynamicRouter#init()}.
     */
    @Test
    public void testInit() {
        try {
            this.dynamicRouter.init();
        } catch (final Throwable t) {
            fail("An error occurs: " + t.getMessage());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#getParamList()
     */
    @Override
    public List<Param> getParamList() {
        final List<Param> paramList = new CopyOnWriteArrayList<Param>();
        Param param = new Param();
        param.setName(EIPConstants.TEST);
        param.setValue("boolean(/*[local-name()='PublishContractResponse' and namespace-uri()='http://www.petals.org/ContractService1'])");
        paramList.add(param);
        param = new Param();
        param.setName(EIPConstants.TEST);
        param.setValue("boolean(/*[local-name()='PublishContractResponse' and namespace-uri()='http://www.petals.org/ContractService2'])");
        paramList.add(param);
        return paramList;
    }

    private List<String> getConditionList() {
        final List<String> conditionList = new CopyOnWriteArrayList<String>();
        for (final Param param : this.getParamList()) {
            if (EIPConstants.TEST.equals(param.getName())) {
                conditionList.add(param.getValue());
            }
        }
        return conditionList;
    }

    private List<Consumes> getConsumesList(final MEPType mepType) {
        final List<Consumes> consumesList = new CopyOnWriteArrayList<Consumes>();
        QName interfaceName = null;
        QName serviceName = null;
        String endpoint = null;

        interfaceName = new QName("http://petals.ow2.org", "interface");
        serviceName = new QName("http://petals.ow2.org", "serviceConsumer");
        endpoint = "8b32ee51-9b7b-4a09-8805-99733c1efc9a";
        Consumes consumes = new Consumes();
        consumes.setMep(mepType);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "router"));
        consumesList.add(consumes);
        consumes = new Consumes();
        consumes.setMep(mepType);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "test1"));
        consumesList.add(consumes);
        consumes = new Consumes();
        consumes.setMep(MEPType.IN_ONLY);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "test2"));
        consumesList.add(consumes);
        consumes = new Consumes();
        consumes.setMep(MEPType.IN_ONLY);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "default"));
        consumesList.add(consumes);
        return consumesList;
    }

}
