/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.jbidescriptor.generated.Param;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.se.eip.EIPConstants;
import org.ow2.petals.se.eip.patterns.mock.MessageExchangeMock;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.XMLComparator;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public class RouterOperationTest extends AbstractMEPBridgeWithUnicConsumesPatternTest {

    private Document in;

    private QName operation1 = new QName("http://petals.ow2.org", "operation1");

    private QName operation2 = new QName("http://petals.ow2.org", "operation2");

    private QName operation3 = new QName("http://petals.ow2.org", "operation3");

    /**
     * @param name
     */
    public RouterOperationTest() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    @Before
    public void setUp() throws Exception {
        super.setUp();
        this.abstractMEPBridgeWithUnicConsumesPattern = new Router();
        
        DocumentBuilder documentBuilder = null;
        InputStream inputStream = null;
        
        try {
            documentBuilder = DocumentBuilders.takeDocumentBuilder();
            
            // in 1
            inputStream = Router.class.getResourceAsStream("/routerIn1.xml");
            this.in = documentBuilder.parse(inputStream);
        } finally {
            if (documentBuilder != null) {
                DocumentBuilders.releaseDocumentBuilder(documentBuilder);
            }
            if (inputStream != null) {
                inputStream.close();
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#tearDown()
     */
    @After
    public void tearDown() throws Exception {
        super.tearDown();
        this.in = null;
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Router#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessTest1DOM() {
        try {
            this.context.setConsumeList(getConsumesList(MEPType.IN_OUT));
            // exchange
            final Exchange exchange = this.processDOMTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, this.operation1, this.in, null, null,
                    null, this.abstractMEPBridgeWithUnicConsumesPattern);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(0)
                            .getInMessage().getContent()))));
            assertEquals("test1", exchangeList.get(0).getOperation().getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Router#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessTest2DOM() {
        try {
            this.context.setConsumeList(getConsumesList(MEPType.IN_OUT));
            // exchange
            final Exchange exchange = this.processDOMTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, this.operation2, this.in, null, null,
                    null, this.abstractMEPBridgeWithUnicConsumesPattern);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(0)
                            .getInMessage().getContent()))));
            assertEquals("test2", exchangeList.get(0).getOperation().getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Router#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessDefaultDOM() {
        try {
            this.context.setConsumeList(getConsumesList(MEPType.IN_OUT));
            // exchange
            final Exchange exchange = this.processDOMTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, this.operation3, this.in, null, null,
                    null, this.abstractMEPBridgeWithUnicConsumesPattern);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(0)
                            .getInMessage().getContent()))));
            assertEquals("default", exchangeList.get(0).getOperation().getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Router#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessTest1Stream() {
        try {
            this.context.setConsumeList(getConsumesList(MEPType.IN_OUT));
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, this.operation1, this.in, null, null,
                    null, this.abstractMEPBridgeWithUnicConsumesPattern);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals("test1", exchangeList.get(0).getOperation().getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Router#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessTest2Stream() {
        try {
            this.context.setConsumeList(getConsumesList(MEPType.IN_OUT));
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, this.operation2, this.in, null, null,
                    null, this.abstractMEPBridgeWithUnicConsumesPattern);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals("test2", exchangeList.get(0).getOperation().getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.Router#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessDefaultStream() {
        try {
            this.context.setConsumeList(getConsumesList(MEPType.IN_OUT));
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, this.operation3, this.in, null, null,
                    null, this.abstractMEPBridgeWithUnicConsumesPattern);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals("default", exchangeList.get(0).getOperation().getLocalPart());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for {@link org.ow2.petals.se.eip.patterns.Router#init()}.
     */
    @Override
    @Test
    public void testInit() {
        try {
            this.abstractMEPBridgeWithUnicConsumesPattern.init();
        } catch (final Throwable t) {
            fail("An error occurs: " + t.getMessage());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#getParamList()
     */
    @Override
    public List<Param> getParamList() {
        final List<Param> paramList = new ArrayList<Param>();
        Param param = new Param();
        param.setName(EIPConstants.TEST_OPERATION);
        param.setValue(operation1.toString());
        paramList.add(param);
        param = new Param();
        param.setName(EIPConstants.TEST_OPERATION);
        param.setValue(operation2.toString());
        paramList.add(param);
        return paramList;
    }

    private static final List<Consumes> getConsumesList(final MEPType mepType) {
        final List<Consumes> consumesList = new ArrayList<Consumes>();

        QName interfaceName = new QName("http://petals.ow2.org", "interface");
        QName serviceName = new QName("http://petals.ow2.org", "serviceConsumer");
        String endpoint = "8b32ee51-9b7b-4a09-8805-99733c1efc9a";
        Consumes consumes = new Consumes();
        consumes.setMep(mepType);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "test1"));
        consumesList.add(consumes);
        consumes = new Consumes();
        consumes.setMep(MEPType.IN_ONLY);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "test2"));
        consumesList.add(consumes);
        consumes = new Consumes();
        consumes.setMep(MEPType.IN_ONLY);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "default"));
        consumesList.add(consumes);
        return consumesList;
    }
}
