/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.jbidescriptor.generated.Param;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.se.eip.async.CommonAsyncContext;
import org.ow2.petals.se.eip.async.RoutingSlipAsyncContext;
import org.ow2.petals.se.eip.patterns.mock.MessageExchangeMock;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.XMLComparator;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public class RoutingSlipTest extends AbstractTest {

    private RoutingSlip routingSlip;

    private Document in1;

    private Document out1;

    private Document out2;

    private Document fault1;

    /**
     * @param name
     */
    public RoutingSlipTest() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#setUp()
     */
    @Before
    public void setUp() throws Exception {
        super.setUp();
        this.routingSlip = new RoutingSlip();
        
        DocumentBuilder documentBuilder = null;
        InputStream inputStream = null;
        
        try {
            documentBuilder = DocumentBuilders.takeDocumentBuilder();

            // in 1
            inputStream = RoutingSlip.class.getResourceAsStream("/routingSlipIn1.xml");
            this.in1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 1
            inputStream = RoutingSlip.class.getResourceAsStream("/routingSlipOut1.xml");
            this.out1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // out 2
            inputStream = RoutingSlip.class.getResourceAsStream("/routingSlipOut2.xml");
            this.out2 = documentBuilder.parse(inputStream);
            inputStream.close();
            // fault 1
            inputStream = RoutingSlip.class.getResourceAsStream("/routingSlipFault1.xml");
            this.fault1 = documentBuilder.parse(inputStream);
        } finally {
            if (documentBuilder != null) {
                DocumentBuilders.releaseDocumentBuilder(documentBuilder);
            }
            if (inputStream != null) {
                inputStream.close();
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#tearDown()
     */
    @After
    public void tearDown() throws Exception {
        super.tearDown();
        this.routingSlip = null;
        this.in1 = null;
        this.out1 = null;
        this.out2 = null;
        this.fault1 = null;
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessDOM() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            // exchange
            final Exchange exchange = this.processDOMTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.routingSlip);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertEquals(1, exchangeList.size());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(0)
                            .getInMessage().getContent()))));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
    public void testProcessStream() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.routingSlip);
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncDOM() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final SourceKind sourceKind = new SourceKind();
            sourceKind.setStreamKind();
            final Exchange originalExchange = this.createExchange(sourceKind,
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            // exchange 1
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncDOMTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(4, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNotNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            assertTrue(XMLComparator.isEquivalent(XMLPrettyPrinter.prettyPrint(this.out2), result));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncStream() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_OUT);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.createExchange(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    this.in1, null, null, null);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(4, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNotNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            assertEquals(XMLPrettyPrinter.prettyPrint(this.out2), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOnlyWithInOnlyConsumesWithDone() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.routingSlip, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // unable to route
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals(
                    "Terminate the pattern processing due to a Done status received before the end of the global routing",
                    eipAsyncContext.getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOnlyWithInOnlyConsumesWithError() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.routingSlip, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOnlyWithRobustInOnlyConsumesWithDone() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.routingSlip, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // unable to route
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals(
                    "Terminate the pattern processing due to a Done status received before the end of the global routing",
                    eipAsyncContext.getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOnlyWithRobustInOnlyConsumesWithError() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.routingSlip, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOnlyWithRobustInOnlyConsumesWithFault() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            // InOnly, so no fault thrown, just exceptions
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOnlyWithInOutConsumesWithOut() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(5, this.context.getExchangeList().size());
            // InOnly, so no out, just exceptions
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOnlyWithInOutConsumesWithFault() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_ONLY_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE,
                    null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            // InOnly, so no fault thrown, just exceptions
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncRobustInOnlyWithInOnlyConsumesWithDone() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.routingSlip, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // unable to route
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals(
                    "Terminate the pattern processing due to a Done status received before the end of the global routing",
                    eipAsyncContext.getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncRobustInOnlyWithInOnlyConsumesWithError() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.routingSlip, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncRobustInOnlyWithRobustInOnlyConsumesWithDone() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.routingSlip, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // unable to route
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals(
                    "Terminate the pattern processing due to a Done status received before the end of the global routing",
                    eipAsyncContext.getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncRobustInOnlyWithRobustInOnlyConsumesWithError() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.routingSlip, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isErrorStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncRobustInOnlyWithRobustInOnlyConsumesWithFault() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNotNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(eipAsyncContext.getOriginalExchange().getFault().getContent()));
            assertEquals(XMLPrettyPrinter.prettyPrint(this.fault1), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncRobustInOnlyWithInOutConsumesWithOut() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(5, this.context.getExchangeList().size());
            // RobustInOnly, so no out, just exceptions
            assertTrue(eipAsyncContext.getOriginalExchange().isDoneStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncRobustInOnlyWithInOutConsumesWithFault() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNotNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(eipAsyncContext.getOriginalExchange().getFault().getContent()));
            assertEquals(XMLPrettyPrinter.prettyPrint(this.fault1), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOutWithInOnlyConsumesWithDone() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.routingSlip, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // unable to route
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            // we can't test if the out message is null. if we invoke the
            // getOutMessage() method...one is created
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals(
                    "Terminate the pattern processing due to a Done status received before the end of the global routing",
                    eipAsyncContext.getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOutWithInOnlyConsumesWithError() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.routingSlip, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            // we can't test if the out message is null. if we invoke the
            // getOutMessage() method...one is created
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOutWithRobustInOnlyConsumesWithDone() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.DONE, null, null, null, null, this.routingSlip, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // unable to route
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            // we can't test if the out message is null. if we invoke the
            // getOutMessage() method...one is created
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals(
                    "Terminate the pattern processing due to a Done status received before the end of the global routing",
                    eipAsyncContext.getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOutWithRobustInOnlyConsumesWithError() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ERROR, null, null, null, new Exception("test exception"),
                    this.routingSlip, eipAsyncContext);
            assertEquals(2, this.context.getExchangeList().size());
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            // we can't test if the out message is null. if we invoke the
            // getOutMessage() method...one is created
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNotNull(eipAsyncContext.getOriginalExchange().getError());
            assertEquals("java.lang.Exception: test exception", eipAsyncContext
                    .getOriginalExchange().getError().getMessage());
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOutWithRobustInOnlyConsumesWithFault() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            // we can't test if the out message is null. if we invoke the
            // getOutMessage() method...one is created
            assertNotNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(eipAsyncContext.getOriginalExchange().getFault().getContent()));
            assertEquals(XMLPrettyPrinter.prettyPrint(this.fault1), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOutWithInOutConsumesWithOut() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out1, null, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            // exchange 2
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, this.out2, null, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(5, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            assertNotNull(eipAsyncContext.getOriginalExchange().getOutMessage());
            assertNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(
                            eipAsyncContext.getOriginalExchange().getOutMessage().getContent()));
            assertEquals(XMLPrettyPrinter.prettyPrint(this.out2), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.RoutingSlip#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)}
     * .
     */
    @Test
    public void testProcessAsyncInOutWithInOutConsumesWithFault() {
        try {
            this.routingSlip.init();
            this.context.getExchangeList().clear();
            final List<Consumes> consumesList = this.getConsumesList(MEPType.IN_ONLY);
            this.context.setConsumeList(consumesList);
            // original exchange
            final Exchange originalExchange = this.processStreamTest(
                    MessageExchangeMock.IN_OUT_PATTERN, Role.PROVIDER, ExchangeStatus.ACTIVE, null,
                    this.in1, null, null, null, this.routingSlip);
            final RoutingSlipAsyncContext eipAsyncContext = new RoutingSlipAsyncContext(
                    originalExchange, CommonAsyncContext.DEFAULT_TIME_TO_LIVE, false, consumesList);
            assertEquals(1, this.context.getExchangeList().size());
            // exchange 1
            this.processAsyncStreamTest(MessageExchangeMock.IN_OUT_PATTERN, Role.CONSUMER,
                    ExchangeStatus.ACTIVE, null, null, this.fault1, null, this.routingSlip,
                    eipAsyncContext);
            assertEquals(3, this.context.getExchangeList().size());
            assertTrue(eipAsyncContext.getOriginalExchange().isActiveStatus());
            // we can't test if the out message is null. if we invoke the
            // getOutMessage() method...one is created
            assertNotNull(eipAsyncContext.getOriginalExchange().getFault());
            assertNull(eipAsyncContext.getOriginalExchange().getError());
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(eipAsyncContext.getOriginalExchange().getFault().getContent()));
            assertEquals(XMLPrettyPrinter.prettyPrint(this.fault1), result);
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for {@link org.ow2.petals.se.eip.patterns.RoutingSlip#init()}
     * .
     */
    @Test
    public void testInit() {
        try {
            this.routingSlip.init();
        } catch (final Throwable t) {
            fail("An error occurs: " + t.getMessage());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#getParamList()
     */
    @Override
    public List<Param> getParamList() {
        return new CopyOnWriteArrayList<Param>();
    }

    private List<Consumes> getConsumesList(final MEPType mepType) {
        final List<Consumes> consumesList = new ArrayList<Consumes>();
        QName interfaceName = new QName("http://petals.ow2.org", "interface");
        QName serviceName = new QName("http://petals.ow2.org", "serviceConsumer");
        String endpoint = "8b32ee51-9b7b-4a09-8805-99733c1efc9a";
        final Consumes consumes = new Consumes();
        consumes.setMep(mepType);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "test1"));
        consumesList.add(consumes);
        consumes.setOperation(new QName("http://petals.ow2.org", "test2"));
        consumesList.add(consumes);
        return consumesList;
    }

}
