/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.MEPType;
import org.ow2.petals.component.framework.jbidescriptor.generated.Param;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.se.eip.EIPConstants;
import org.ow2.petals.se.eip.patterns.mock.ExchangeContextMock;
import org.ow2.petals.se.eip.patterns.mock.MessageExchangeMock;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.XMLComparator;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public class WireTapTest extends AbstractTest {

    private WireTap wireTap;

    private Document in1;

    private Document in2;

    /**
     * @param name
     */
    public WireTapTest() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    @Before
    public void setUp() throws Exception {
        super.setUp();
        this.wireTap = new WireTap();
        
        DocumentBuilder documentBuilder = null;
        InputStream inputStream = null;
        
        try {
            documentBuilder = DocumentBuilders.takeDocumentBuilder();
            
            // in 1
            inputStream = WireTap.class.getResourceAsStream("/wireTapIn1.xml");
            this.in1 = documentBuilder.parse(inputStream);
            inputStream.close();
            // in 2
            inputStream = WireTap.class.getResourceAsStream("/wireTapIn2.xml");
            this.in2 = documentBuilder.parse(inputStream);
        } finally {
            if(documentBuilder != null) {
                DocumentBuilders.releaseDocumentBuilder(documentBuilder);
            }
            if(inputStream != null) {
                inputStream.close();
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#tearDown()
     */
    @After
    public void tearDown() throws Exception {
        super.tearDown();
        this.wireTap = null;
        this.in1 = null;
        this.in2 = null;
    }

    /**
     * Test method for {@link org.ow2.petals.se.eip.patterns.WireTap#init()}.
     */
    @Test
 	public void testInit() {
        try {
            this.wireTap.init();
        } catch (final Throwable t) {
            fail("An error occurs: " + t.getMessage());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.se.eip.patterns.AbstractTest#getParamList()
     */
    @Override
    public List<Param> getParamList() {
        return null;
    }

    private List<Param> getParamList(final String way) {
        final List<Param> paramList = new ArrayList<Param>();
        final Param param = new Param();
        param.setName(EIPConstants.WIRE_TAP_WAY);
        param.setValue(way);
        paramList.add(param);
        return paramList;
    }

    private void createContext(final String way) {
        this.context = new ExchangeContextMock(new ConfigurationExtensions(this.getParamList(way)));
    }

    private List<Consumes> getConsumesList(final MEPType mepType) {
        final List<Consumes> consumesList = new CopyOnWriteArrayList<Consumes>();
        QName interfaceName = null;
        QName serviceName = null;
        String endpoint = null;

        interfaceName = new QName("http://petals.ow2.org", "interface");
        serviceName = new QName("http://petals.ow2.org", "serviceConsumer");
        endpoint = "8b32ee51-9b7b-4a09-8805-99733c1efc9a";
        Consumes consumes = new Consumes();
        consumes.setMep(mepType);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "test1"));
        consumesList.add(consumes);
        consumes = new Consumes();
        consumes.setMep(MEPType.IN_ONLY);
        consumes.setInterfaceName(interfaceName);
        consumes.setServiceName(serviceName);
        consumes.setEndpointName(endpoint);
        consumes.setOperation(new QName("http://petals.ow2.org", "test2"));
        consumesList.add(consumes);
        return consumesList;
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessDOM() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processDOMTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertNotNull(exchange.getOutMessage());
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(0)
                            .getInMessage().getContent()))));
            assertTrue(XMLComparator.isEquivalent(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(exchangeList.get(1)
                            .getInMessage().getContent()))));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessStream() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());
            assertNotNull(exchange.getOutMessage());
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestWithInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyResponseWithInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestResponseWithInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestOnResponseWithInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestWithInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyResponseWithInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestResponseWithInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestOnResponseWithInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestWithRobustInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyResponseWithRobustInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestResponseWithRobustInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestOnResponseWithRobustInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestWithRobustInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyResponseWithRobustInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestResponseWithRobustInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestOnResponseWithRobustInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestWithRobustInOnlyConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyResponseWithRobustInOnlyConsumesWithFault() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestResponseWithRobustInOnlyConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(4, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestOnResponseWithRobustInOnlyConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyResponseWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestResponseWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(4, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestOnResponseWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestWithInOutConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyResponseWithInOutConsumesWithFault() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestResponseWithInOutConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(4, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOnlyRequestOnResponseWithInOutConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_ONLY_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestWithInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyResponseWithInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestResponseWithInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestOnResponseWithInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestWithInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyResponseWithInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestResponseWithInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestOnResponseWithInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestWithRobustInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyResponseWithRobustInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestResponseWithRobustInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestOnResponseWithRobustInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestWithRobustInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyResponseWithRobustInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestResponseWithRobustInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestOnResponseWithRobustInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isErrorStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestWithRobustInOnlyConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyResponseWithRobustInOnlyConsumesWithFault() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestResponseWithRobustInOnlyConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(4, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestOnResponseWithRobustInOnlyConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyResponseWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestResponseWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(4, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestOnResponseWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestWithInOutConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyResponseWithInOutConsumesWithFault() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestResponseWithInOutConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(4, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessRobustInOnlyRequestOnResponseWithInOutConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(
                    MessageExchangeMock.ROBUST_IN_ONLY_PATTERN, Role.PROVIDER,
                    ExchangeStatus.ACTIVE, null, this.in1, null, null, null, this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestWithInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutResponseWithInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestResponseWithInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestOnResponseWithInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestWithInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutResponseWithInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestResponseWithInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestOnResponseWithInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestWithRobustInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutResponseWithRobustInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestResponseWithRobustInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestOnResponseWithRobustInOnlyConsumesWithDone() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestWithRobustInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));

        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessinOutResponseWithRobustInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestResponseWithRobustInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestOnResponseWithRobustInOnlyConsumesWithError() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(false);
            this.context.setError(true);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            // the exchange must be on active status with an error: it's out of
            // the
            // JBI specification, but managed further by the message exchange
            // processor
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNotNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(1, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestWithRobustInOnlyConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutResponseWithRobustInOnlyConsumesWithFault() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestResponseWithRobustInOnlyConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(4, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestOnResponseWithRobustInOnlyConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutResponseWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestResponseWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(4, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestOnResponseWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestWithInOutConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutResponseWithInOutConsumesWithFault() {
        try {
            this.createContext(WireTap.RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestResponseWithInOutConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OUT));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(4, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOutRequestOnResponseWithInOutConsumesWithFault() {
        try {
            this.createContext(WireTap.REQUEST_ON_RESPONSE_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.ROBUST_IN_ONLY));
            this.context.setFault(true);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in1, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNotNull(exchange.getFault());
            assertNull(exchange.getError());
            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(2, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in1),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }

    /**
     * Test method for
     * {@link org.ow2.petals.se.eip.patterns.WireTap#process(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext)}
     * .
     */
    @Test
 	public void testProcessInOptionalOutRequestWithInOutConsumesWithOut() {
        try {
            this.createContext(WireTap.REQUEST_WAY);
            this.wireTap.init();
            this.context.getExchangeList().clear();
            this.context.setConsumeList(this.getConsumesList(MEPType.IN_OPTIONAL_OUT));
            this.context.setFault(false);
            this.context.setError(false);
            // exchange
            final Exchange exchange = this.processStreamTest(MessageExchangeMock.IN_OUT_PATTERN,
                    Role.PROVIDER, ExchangeStatus.ACTIVE, null, this.in2, null, null, null,
                    this.wireTap);
            assertTrue(exchange.isActiveStatus());
            assertNull(exchange.getFault());
            assertNull(exchange.getError());

            final List<Exchange> exchangeList = this.context.getExchangeList();
            assertEquals(3, exchangeList.size());
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in2),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(0).getInMessage().getContent())));
            assertEquals(
                    XMLPrettyPrinter.prettyPrint(this.in2),
                    XMLPrettyPrinter.prettyPrint(SourceUtil.createDocument(
                            exchangeList.get(1).getInMessage().getContent())));
            final String result = XMLPrettyPrinter.prettyPrint(SourceUtil
                    .createDocument(exchange.getOutMessage().getContent()));
            assertNotNull(result);
            assertTrue(XMLComparator
                    .isEquivalent(
                            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<content>The context set an out</content>\n",
                            result));
        } catch (final PEtALSCDKException e) {
            fail("An error occurs: " + e.getMessage());
        } catch (final MessagingException e) {
            fail("An error occurs: " + e.getMessage());
        }
    }
}
