/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns.mock;

import java.io.InputStream;
import java.net.URI;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.security.auth.Subject;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.apache.commons.lang.NotImplementedException;
import org.ow2.petals.commons.log.FlowAttributes;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.message.ExchangeImpl;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.w3c.dom.Document;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public class ExchangeImplMock implements Exchange {

    private final MessageExchange exchange;

    private NormalizedMessage inNormalizedExchange;

    private NormalizedMessage outNormalizedExchange;

    private boolean checkRole;

    private boolean checkStatus;

    /**
     * 
     * Creates a new instance of {@link ExchangeImpl}
     * 
     * @param exchange
     */
    public ExchangeImplMock(final MessageExchange exchange) {
        super();
        this.exchange = exchange;
        this.inNormalizedExchange = this.exchange.getMessage(IN_MESSAGE_NAME);
        this.outNormalizedExchange = this.exchange.getMessage(OUT_MESSAGE_NAME);
        this.checkRole = true;
        this.checkStatus = true;
    }

    /**
     * Creates a message.
     * <p>
     * This method is not exposed by the {@code Exchange} interface.
     * 
     * @return
     * @throws MessagingException
     */
    public NormalizedMessage createMessage() throws MessagingException {
        return this.exchange.createMessage();
    }

    /**
     * Return the wrapped Message Exchange.
     * <p>
     * This method is not exposed by the {@code Exchange} interface.
     * 
     * @return the wrapped Message Exchange
     */
    public MessageExchange getMessageExchange() {
        try {
            if (this.inNormalizedExchange != null
                    && this.exchange.getMessage(IN_MESSAGE_NAME) == null) {
                this.exchange.setMessage(this.inNormalizedExchange, IN_MESSAGE_NAME);
            }

            if (this.outNormalizedExchange != null
                    && this.exchange.getMessage(OUT_MESSAGE_NAME) == null) {
                this.exchange.setMessage(this.outNormalizedExchange, OUT_MESSAGE_NAME);
            }
        } catch (final MessagingException e) {
            // do nothing
        }

        return this.exchange;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#createFault()
     */
    public Fault createFault() throws MessagingException {
        return this.exchange.createFault();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getEndpoint()
     */
    public ServiceEndpoint getEndpoint() {
        return this.exchange.getEndpoint();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getError()
     */
    public Exception getError() {
        return this.exchange.getError();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getExchangeId()
     */
    public String getExchangeId() {
        return this.exchange.getExchangeId();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getFault()
     */
    public Fault getFault() {
        return this.exchange.getFault();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getInterfaceName
     * ()
     */
    public QName getInterfaceName() {
        return this.exchange.getInterfaceName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getOperation()
     */
    public QName getOperation() {
        return this.exchange.getOperation();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getPattern()
     */
    public URI getPattern() {
        return this.exchange.getPattern();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getProperty(java
     * .lang.String)
     */
    public Object getProperty(final String name) {
        return this.exchange.getProperty(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getPropertyNames
     * ()
     */
    public Set<?> getPropertyNames() {
        return this.exchange.getPropertyNames();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getRole()
     */
    public MessageExchange.Role getRole() {
        return this.exchange.getRole();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getService()
     */
    public QName getService() {
        return this.exchange.getService();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getStatus()
     */
    public ExchangeStatus getStatus() {
        return this.exchange.getStatus();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isTransacted()
     */
    public boolean isTransacted() {
        return this.exchange.isTransacted();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setEndpoint(javax
     * .jbi.servicedesc.ServiceEndpoint)
     */
    public void setEndpoint(final ServiceEndpoint endpoint) {
        this.exchange.setEndpoint(endpoint);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setError(java
     * .lang.Exception)
     */
    public void setError(final Exception error) {
        final String packageName = error.getClass().getPackage().getName();
        if (packageName.startsWith("java") || packageName.startsWith("org.w3c")
                || packageName.startsWith("org.xml")) {
            this.exchange.setError(error);
        } else {
            throw new IllegalArgumentException("Exception of type '" + packageName
                    + "' are not supported by the JBI container");
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setFault(javax
     * .jbi.messaging.Fault)
     */
    public void setFault(final Fault fault) throws MessagingException {
        this.exchange.setFault(fault);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setFault(java
     * .lang.Exception)
     */
    public void setFault(final Throwable e) throws MessagingException {
        final Fault fault = this.exchange.createFault();
        final Document faultDocument = SourceUtil.createSoapFaultDocument(e,
                Exchange.DEFAULT_FAULT_QNAME);
        if (SourceUtil.isDOM3(faultDocument)) {
            fault.setContent(SourceUtil.createDOMSource(faultDocument));
        } else {
            try {
                fault.setContent(SourceUtil.createStreamSource(faultDocument));
            } catch (final PEtALSCDKException e1) {
                throw new MessagingException(e1);
            }
        }
        this.exchange.setFault(fault);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInterfaceName
     * (javax.xml.namespace.QName)
     */
    public void setInterfaceName(final QName interfaceName) {
        this.exchange.setInterfaceName(interfaceName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setOperation(
     * javax.xml.namespace.QName)
     */
    public void setOperation(final QName name) {
        this.exchange.setOperation(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setProperty(java
     * .lang.String, java.lang.Object)
     */
    public void setProperty(final String name, final Object obj) {
        this.exchange.setProperty(name, obj);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setService(javax
     * .xml.namespace.QName)
     */
    public void setService(final QName service) {
        this.exchange.setService(service);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setStatus(javax
     * .jbi.messaging.ExchangeStatus)
     */
    public void setStatus(final ExchangeStatus status) throws MessagingException {
        this.checkStatus(status);

        this.exchange.setStatus(status);
    }

    /**
     * Copy attachement to a normalized message.
     * 
     * @param attachments
     *            Attachments to copy. Must be not-null.
     * @param normalizedMesg
     *            Normalized message in which attachments must be copied. Must
     *            be not-null.
     * @throws MessagingException
     *             An error occurs when adding an attachement to the normalized
     *             message.
     */
    final private static void copyAttachements(final Map<String, DataHandler> attachments,
            final NormalizedMessage normalizedMesg) throws MessagingException {
        for (final Map.Entry<String, DataHandler> entry : attachments.entrySet()) {
            normalizedMesg.addAttachment(entry.getKey(), entry.getValue());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setOutMessageAttachments(java.util.Map)
     */
    public void setOutMessageAttachments(final Map<String, DataHandler> attachments)
    throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        copyAttachements(attachments, outMess);
        this.setOutMessage(outMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setInMessageAttachments(java.util.Map)
     */
    public void setInMessageAttachments(final Map<String, DataHandler> attachments)
    throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        copyAttachements(attachments, inMess);
        this.setInMessage(inMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessageAttachments()
     */
    public Set<DataHandler> getInMessageAttachments() throws MessagingException {
        final Set<DataHandler> attachments = new HashSet<DataHandler>();

        final NormalizedMessage inMessage = this.getInMessage();
        if (inMessage == null) {
            throw new MessagingException("In message cannot be null");
        }

        final Set<?> names = inMessage.getAttachmentNames();
        for (final Object attachId : names) {
            final DataHandler dh = inMessage.getAttachment((String) attachId);
            if (dh != null) {
                attachments.add(dh);
            }
        }
        return attachments;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getExchangePattern
     * ()
     */
    public URI getExchangePattern() throws MessagingException {
        final URI pattern = this.exchange.getPattern();
        if (pattern == null) {
            throw new MessagingException(
            "You must specify a message exchange pattern in the message exchange");
        }
        return pattern;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getOperationName
     * ()
     */
    public String getOperationName() {
        final QName operation = this.exchange.getOperation();
        if (operation == null) {
            return null;
        }
        return operation.getLocalPart();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getEndpointName()
     */
    public String getEndpointName() throws MessagingException {
        final ServiceEndpoint ep = this.exchange.getEndpoint();
        if (ep == null) {
            throw new MessagingException("You must specify an endpoint in the message exchange");
        }
        return ep.getEndpointName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getInMessage()
     */
    public NormalizedMessage getInMessage() {
        if (this.inNormalizedExchange == null) {
            if (this.exchange.getMessage(IN_MESSAGE_NAME) != null) {
                return this.exchange.getMessage(IN_MESSAGE_NAME);
            }
            try {
                this.inNormalizedExchange = this.exchange.createMessage();
                this.exchange.setMessage(this.inNormalizedExchange, IN_MESSAGE_NAME);
            } catch (final MessagingException e) {
                return null;
            }
        }
        return this.inNormalizedExchange;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isInMessage()
     */
    public boolean isInMessage() {
        return this.exchange.getMessage(IN_MESSAGE_NAME) != null
        && this.exchange.getMessage(IN_MESSAGE_NAME).getContent() != null
        || this.inNormalizedExchange != null
        && this.inNormalizedExchange.getContent() != null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInMessage(
     * javax.jbi.messaging.NormalizedMessage)
     */
    public void setInMessage(final NormalizedMessage msg) throws MessagingException {
        this.checkPatternMatching(IN_MESSAGE_NAME);
        this.checkRoleMatching(IN_MESSAGE_NAME);
        this.inNormalizedExchange = msg;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getOutMessage()
     */
    public NormalizedMessage getOutMessage() {
        if (this.outNormalizedExchange == null) {
            if (this.exchange.getMessage(OUT_MESSAGE_NAME) != null) {
                return this.exchange.getMessage(OUT_MESSAGE_NAME);
            } else {
                try {
                    this.outNormalizedExchange = this.exchange.createMessage();
                    this.exchange.setMessage(this.outNormalizedExchange, OUT_MESSAGE_NAME);
                } catch (final MessagingException e) {
                    return null;
                }
            }
        }
        return this.outNormalizedExchange;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isOutMessage()
     */
    public boolean isOutMessage() {
        return this.exchange.getMessage(OUT_MESSAGE_NAME) != null
        && this.exchange.getMessage(OUT_MESSAGE_NAME).getContent() != null
        || this.outNormalizedExchange != null
        && this.outNormalizedExchange.getContent() != null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setOutMessage
     * (javax.jbi.messaging.NormalizedMessage)
     */
    public void setOutMessage(final NormalizedMessage msg) throws MessagingException {
        this.checkPatternMatching(OUT_MESSAGE_NAME);
        this.checkRoleMatching(OUT_MESSAGE_NAME);
        this.outNormalizedExchange = msg;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setDoneStatus()
     */
    public void setDoneStatus() throws MessagingException {
        this.setStatus(ExchangeStatus.DONE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isDoneStatus()
     */
    public boolean isDoneStatus() {
        return ExchangeStatus.DONE.equals(this.getStatus());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setErrorStatus()
     */
    public void setErrorStatus() throws MessagingException {
        this.setStatus(ExchangeStatus.ERROR);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isErrorStatus()
     */
    public boolean isErrorStatus() {
        return ExchangeStatus.ERROR.equals(this.getStatus());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setActiveStatus()
     */
    public void setActiveStatus() throws MessagingException {
        this.setStatus(ExchangeStatus.ACTIVE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isActiveStatus()
     */
    public boolean isActiveStatus() {
        return ExchangeStatus.ACTIVE.equals(this.getStatus());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isProviderRole()
     */
    public boolean isProviderRole() {
        return MessageExchange.Role.PROVIDER.equals(this.getRole());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isConsumerRole()
     */
    public boolean isConsumerRole() {
        return MessageExchange.Role.CONSUMER.equals(this.getRole());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isInOnlyPattern()
     */
    public boolean isInOnlyPattern() {
        return MEPConstants.IN_ONLY_PATTERN.equals(this.exchange.getPattern());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isRobustInOnlyPattern
     * ()
     */
    public boolean isRobustInOnlyPattern() {
        return MEPConstants.ROBUST_IN_ONLY_PATTERN.equals(this.exchange.getPattern());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#isInOutPattern()
     */
    public boolean isInOutPattern() {
        return MEPConstants.IN_OUT_PATTERN.equals(this.exchange.getPattern());
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * isInOptionalOutPattern()
     */
    public boolean isInOptionalOutPattern() {
        return MEPConstants.IN_OPTIONAL_OUT_PATTERN.equals(this.exchange.getPattern());
    }

    /**
     * Set the checkRole
     * 
     * @param checkRole
     *            the checkRole to set
     */
    public void setCheckRole(final boolean checkRole) {
        this.checkRole = checkRole;
        ((MessageExchangeMock) this.exchange).setCheckRole(checkRole);
    }

    /**
     * Set the checkStatus
     * 
     * @param checkStatus
     *            the checkStatus to set
     */
    public void setCheckStatus(final boolean checkStatus) {
        this.checkStatus = checkStatus;
    }

    /**
     * Check that the exchange MEP allows the specified reference. This only
     * works for the JBI predefined MEPs. It check that reference is IN or OUT.
     * Other reference or MEP will cause an exception
     * 
     * @param name
     * @throws MessagingException
     *             the MEP does not allow the reference
     */
    protected void checkPatternMatching(final String name) throws MessagingException {
        if (name == null) {
            throw new MessagingException("The reference name is not defined.");
        }

        if (OUT_MESSAGE_NAME.equalsIgnoreCase(name)
                && (MEPConstants.IN_ONLY_PATTERN.equals(this.exchange.getPattern()) || MEPConstants.ROBUST_IN_ONLY_PATTERN
                        .equals(this.exchange.getPattern()))) {
            throw new MessagingException("the MessageExchange state does not allow this operation.");
        }
    }

    /**
     * Check that the Role allows the specified reference. This only works for
     * the JBI predefined MEPs. It check that reference is IN or OUT and the
     * Role is CONSUMER or PROVIDER. Other reference will cause an exception
     * 
     * @param name
     * @throws MessagingException
     *             the Role does not allow the reference
     */
    protected void checkRoleMatching(final String name) throws MessagingException {
        if (name == null) {
            throw new MessagingException("The reference name is not defined.");
        }

        // The check on the role can be unactivated for CDK intercepting purpose
        if (!this.checkRole) {
            return;
        }

        if (MessageExchange.Role.CONSUMER.equals(this.exchange.getRole())) {
            if (IN_MESSAGE_NAME.equalsIgnoreCase(name)) {
                return;
            }
        } else if (MessageExchange.Role.PROVIDER.equals(this.exchange.getRole())
                && OUT_MESSAGE_NAME.equalsIgnoreCase(name)) {
            return;
        }
        throw new MessagingException("The Role does not allow this operation.");
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getInMessageProperty
     * (java.lang.String)
     */
    public Object getInMessageProperty(final String name) throws MessagingException {
        final NormalizedMessage nm = this.getInMessage();
        if (nm == null) {
            throw new MessagingException("In message cannot be null");
        }
        return nm.getProperty(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getOutMessageProperty
     * (java.lang.String)
     */
    public Object getOutMessageProperty(final String name) throws MessagingException {
        final NormalizedMessage nm = this.getOutMessage();
        if (nm == null) {
            throw new MessagingException("Out message cannot be null");
        }
        return nm.getProperty(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getOutMessagePropertyNames()
     */
    public Set<?> getOutMessagePropertyNames() throws MessagingException {
        final NormalizedMessage nm = this.getOutMessage();
        if (nm == null) {
            throw new MessagingException("Out message cannot be null");
        }
        return nm.getPropertyNames();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInMessageProperty
     * (java.lang.String, java.lang.Object)
     */
    public void setInMessageProperty(final String name, final Object value)
    throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        inMess.setProperty(name, value);
        this.setInMessage(inMess);
    }

    /**
     * Copy properties to a normalized message.
     * 
     * @param properties
     *            Properties to copy. Must be not-null.
     * @param normalizedMesg
     *            Normalized message in which properties must be copied. Must be
     *            not-null.
     */
    final private static void copyProperties(final Map<String, Object> properties,
            final NormalizedMessage normalizedMsg) {
        final Iterator<Map.Entry<String, Object>> iter = properties.entrySet().iterator();
        while (iter.hasNext()) {
            final Map.Entry<String, Object> entry = iter.next();
            normalizedMsg.setProperty(entry.getKey(), entry.getValue());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setInMessageProperties(java.util.Map)
     */
    public void setInMessageProperties(final Map<String, Object> properties)
    throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        copyProperties(properties, inMess);
        this.setInMessage(inMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setOutMessageProperties(java.util.Map)
     */
    public void setOutMessageProperties(final Map<String, Object> properties)
    throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        copyProperties(properties, outMess);
        this.setOutMessage(outMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setOutMessageProperty
     * (java.lang.String, java.lang.Object)
     */
    public void setOutMessageProperty(final String name, final Object value)
    throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        outMess.setProperty(name, value);
        this.setOutMessage(outMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setOutMessageAttachment(java.lang.String, javax.activation.DataHandler)
     */
    public void addOutMessageAttachment(final String id, final DataHandler content)
    throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        outMess.addAttachment(id, content);
        this.setOutMessage(outMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setInMessageAttachment(java.lang.String, javax.activation.DataHandler)
     */
    public void addInMessageAttachment(final String id, final DataHandler content)
    throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        inMess.addAttachment(id, content);
        this.setInMessage(inMess);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInSubject(
     * javax.security.auth.Subject)
     */
    public void setInSubject(final Subject subject) {
        this.getInMessage().setSecuritySubject(subject);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInSubject(
     * javax.security.auth.Subject)
     */
    public void setOutSubject(final Subject subject) {
        this.getOutMessage().setSecuritySubject(subject);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getInAddressing()
     */
    public Map<QName, String> getInAddressing() {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInAddressing
     * (java.util.Map)
     */
    public void setInAddressing(final Map<QName, String> addressing) {
        throw new NotImplementedException();
    }

    /**
     * Check the status against the cached Normalized messages.
     * 
     * @param status
     * @throws MessagingException
     */
    private void checkStatus(final ExchangeStatus status) throws MessagingException {
        if (this.checkStatus) {
            // check that DONE can be set.
            if (ExchangeStatus.DONE.equals(status)) {
                if (Role.CONSUMER.equals(this.exchange.getRole())) {
                    if (MEPConstants.IN_OUT_PATTERN.equals(this.exchange.getPattern())
                            && this.getFault() == null
                            && this.exchange.getMessage(OUT_MESSAGE_NAME) == null) {
                        throw new MessagingException(
                        "The MessageExchange state does not allow this operation.");
                    }
                } else {
                    if (MEPConstants.ROBUST_IN_ONLY_PATTERN.equals(this.exchange.getPattern())
                            && this.getFault() != null) {
                        throw new MessagingException(
                        "The MessageExchange state does not allow this operation.");
                    } else if (MEPConstants.IN_OUT_PATTERN.equals(this.exchange.getPattern())) {
                        throw new MessagingException(
                        "The MessageExchange state does not allow this operation.");
                    } else if (MEPConstants.IN_OPTIONAL_OUT_PATTERN.equals(this.exchange
                            .getPattern())) {
                        if (this.exchange.getFault() != null
                                && this.exchange.getMessage(OUT_MESSAGE_NAME) == null) {
                            throw new MessagingException(
                            "The MessageExchange state does not allow this operation.");
                        } else if (this.exchange.getFault() == null
                                && this.exchange.getMessage(OUT_MESSAGE_NAME) != null) {
                            throw new MessagingException(
                            "The MessageExchange state does not allow this operation.");
                        }
                    }
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessageAttachment(java.lang.String)
     */
    public DataHandler getInMessageAttachment(final String name) throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessageAttachmentNames()
     */
    public Set<String> getInMessageAttachmentNames() throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getInSubject()
     */
    public Subject getInSubject() throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getOutMessageAttachment(java.lang.String)
     */
    public DataHandler getOutMessageAttachment(final String name) throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getOutMessageAttachmentNames()
     */
    public Set<String> getOutMessageAttachmentNames() throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getOutMessageAttachments()
     */
    public Set<DataHandler> getOutMessageAttachments() throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#getOutSubject()
     */
    public Subject getOutSubject() throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setInMessageAttachment(java.lang.String, javax.activation.DataHandler)
     */
    public void setInMessageAttachment(final String name, final DataHandler attachement)
    throws MessagingException {
        throw new NotImplementedException();

    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * setOutMessageAttachment(java.lang.String, javax.activation.DataHandler)
     */
    public void setOutMessageAttachment(final String name, final DataHandler attachment)
    throws MessagingException {
        throw new NotImplementedException();

    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessagePropertyNames()
     */
    public Set<String> getInMessagePropertyNames() throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessageContentAsDocument()
     */
    public Document getInMessageContentAsDocument() throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessageContentAsDocument(boolean)
     */
    public Document getInMessageContentAsDocument(final boolean forkSourceStream)
    throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getInMessageContentAsSource()
     */
    public Source getInMessageContentAsSource() throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getOutMessageContentAsDocument()
     */
    public Document getOutMessageContentAsDocument() throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getOutMessageContentAsDocument(boolean)
     */
    public Document getOutMessageContentAsDocument(final boolean forkSourceStream)
    throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.message.Exchange#
     * getOutMessageContentAsSource()
     */
    public Source getOutMessageContentAsSource() throws MessagingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInMessageContent
     * (org.w3c.dom.Document)
     */
    public void setInMessageContent(final Document inContent) throws MessagingException {
        throw new NotImplementedException();

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInMessageContent
     * (java.io.InputStream)
     */
    public void setInMessageContent(final InputStream inContent) throws MessagingException {
        throw new NotImplementedException();

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setInMessageContent
     * (javax.xml.transform.Source)
     */
    public void setInMessageContent(final Source inContent) throws MessagingException {
        throw new NotImplementedException();

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setOutMessageContent
     * (org.w3c.dom.Document)
     */
    public void setOutMessageContent(final Document outContent) throws MessagingException {
        throw new NotImplementedException();

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setOutMessageContent
     * (java.io.InputStream)
     */
    public void setOutMessageContent(final InputStream inContent) throws MessagingException {
        throw new NotImplementedException();

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.message.Exchange#setOutMessageContent
     * (javax.xml.transform.Source)
     */
    public void setOutMessageContent(final Source outContent) throws MessagingException {
        throw new NotImplementedException();

    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#getFlowAttributes()
     */
    @Override
    public FlowAttributes getFlowAttributes() {
        throw new NotImplementedException();
    }
}
