/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns.mock;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.security.auth.Subject;
import javax.xml.transform.Source;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public class NormalizedMessageMock implements NormalizedMessage {

    /**
     * The message exchange property name to set the compression of the exchange
     * payload.
     */
    public final static String PROPERTY_COMPRESSION = "org.ow2.petals.transport.compress";

    /**
     * The unique serial UID.
     */
    private static final long serialVersionUID = 45438732L;

    /**
     * Attachments to the main message: id + content.
     */
    protected transient Map<String, DataHandler> attachments;

    /**
     * The XML "payload" message.
     */
    protected transient Source content;

    /**
     * Metadata associated with the message content. Map: property + value.
     */
    protected Map<String, Object> properties;

    /**
     * JAAS Security Subject.
     */
    protected Subject subject;

    /**
     * This default constructor creates a new empty
     * <code>NormalizedMessage</code>.
     */
    public NormalizedMessageMock() {
        super();

        this.content = null;
        this.properties = new HashMap<String, Object>();
        this.attachments = new HashMap<String, DataHandler>();
    }

    /*
     * ---------------------------------------------------------------------- N
     * N o r m a l i z e d M e s s a g e M e t h o d s
     * ----------------------------------------------------------------------
     */

    /**
     * @inheritDoc
     */
    public void addAttachment(final String id, final DataHandler attachment) throws MessagingException {
        this.attachments.put(id, attachment);
    }

    /**
     * @inheritDoc
     */
    public DataHandler getAttachment(final String id) {
        return this.attachments.get(id);
    }

    /**
     * @inheritDoc
     */
    public Set<?> getAttachmentNames() {
        return this.attachments.keySet();
    }

    /**
     * @inheritDoc
     */
    public Source getContent() {
        return this.content;
    }

    /**
     * @inheritDoc
     */
    public Object getProperty(final String name) {
        return this.properties.get(name);
    }

    /**
     * @inheritDoc
     */
    public Set<?> getPropertyNames() {
        return this.properties.keySet();
    }

    /**
     * @inheritDoc
     */
    public Subject getSecuritySubject() {
        return this.subject;
    }

    /**
     * @inheritDoc
     */
    public void removeAttachment(final String id) throws MessagingException {
        final Object att = this.attachments.remove(id);

        if (att == null) {
            throw new MessagingException(id + " attachment does not exist.");
        }
    }

    /**
     * @inheritDoc
     */
    public void setContent(final Source content) throws MessagingException {
        this.content = content;
    }

    /**
     * @inheritDoc
     */
    public void setProperty(final String name, final Object value) {
        if (value == null) {
            // case to remove properties not defined (null)
            this.properties.remove(name);

        } else {
            // define this property name with a new value
            this.properties.put(name, value);
        }
    }

    /**
     * @inheritDoc
     */
    public void setSecuritySubject(final Subject securitySubject) {
        this.subject = securitySubject;
    }
}
