/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.eip.patterns.mock;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * @author Frederic Gardes - EBM WebSourcing
 */
public class ServiceEndpointMock implements ServiceEndpoint {

    private String endpointName = null;

    private QName[] interfaces = null;

    private QName serviceName = null;

    private static final String EPR_NAMESPACE = "http://java.sun.com/xml/ns/jbi";

    public ServiceEndpointMock(final QName[] interfaces, final QName serviceName, final String endpointName) {
        super();
        this.interfaces = interfaces;
        this.serviceName = serviceName;
        this.endpointName = endpointName;
    }

    /* (non-Javadoc)
     * @see javax.jbi.servicedesc.ServiceEndpoint#getAsReference(javax.xml.namespace.QName)
     */
    public DocumentFragment getAsReference(final QName operationName) {
        DocumentFragment result = null;

        try {
            final String reference = "<end-point-reference xmlns='" + EPR_NAMESPACE + "' service-name='"
            + this.serviceName.getLocalPart() + "' end-point-name='" + this.endpointName
            + "'></end-point-reference>";
            final InputSource source = this.createSource(reference);
            final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder;
            builder = factory.newDocumentBuilder();
            final Document document = builder.parse(source);
            result = document.createDocumentFragment();
            builder.reset();
        } catch (final ParserConfigurationException e) {
            // nothing to do
        } catch (final SAXException e) {
            // nothing to do
        } catch (final IOException e) {
            // nothing to do
        }
        return result;
    }

    /* (non-Javadoc)
     * @see javax.jbi.servicedesc.ServiceEndpoint#getEndpointName()
     */
    public String getEndpointName() {
        return this.endpointName;
    }

    /* (non-Javadoc)
     * @see javax.jbi.servicedesc.ServiceEndpoint#getInterfaces()
     */
    public QName[] getInterfaces() {
        return this.interfaces;
    }

    /* (non-Javadoc)
     * @see javax.jbi.servicedesc.ServiceEndpoint#getServiceName()
     */
    public QName getServiceName() {
        return this.serviceName;
    }

    private InputSource createSource(final String msg) {
        final InputSource source = new InputSource();
        final byte[] msgByte = msg.getBytes();
        final ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
        source.setByteStream(in);
        return source;
    }
}
